<?php

namespace App\Importer;

use App\Helpers\Helper;
use App\Models\Consumable;

class ConsumableImporter extends ItemImporter
{
    protected $consumables;
    public function __construct($filename)
    {
        parent::__construct($filename);
        $this->consumables = Consumable::all();
    }

    protected function handle($row)
    {
        parent::handle($row); // TODO: Change the autogenerated stub
        $this->createConsumableIfNotExists();
    }

    /**
     * Create a consumable if a duplicate does not exist
     *
     * @author Daniel Melzter
     * @since 3.0
     */
    public function createConsumableIfNotExists()
    {
        $consumableId = $this->consumables->search(function ($key) {
            return strcasecmp($key->name, $this->item['name']) == 0;
        });
        if ($consumableId !== false) {
            if (!$this->updating) {
                $this->log('A matching Consumable ' . $this->item["name"] . ' already exists.  ');
                return;
            }
            $this->log('Updating Consumable');
            $consumable = $this->consumables[$consumableId];
            $consumable->update($this->sanitizeItemForUpdating($consumable));
            if (!$this->testRun) {
                $consumable->save();
            }
            return;
        }
        $this->log("No matching consumable, creating one");
        $consumable = new Consumable();
        $consumable->fill($this->sanitizeItemForStoring($consumable));

        if ($this->testRun) {
            $this->log('TEST RUN - Consumable ' . $this->item['name'] . ' not created');
            return;
        }
        if ($consumable->save()) {
            $consumable->logCreate('Imported using CSV Importer');
            $this->log("Consumable " . $this->item["name"] . ' was created');
            return;
        }
        $this->jsonError($consumable, 'Consumable');
        return;
    }
}
